"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.handler = exports.help = exports.cliOptionsTypes = exports.rcOptionsTypes = exports.commandNames = exports.INDEX_PATH_CLR = exports.PACKAGE_INFO_CLR = void 0;
const path_1 = __importDefault(require("path"));
const fs_1 = __importDefault(require("fs"));
const chalk_1 = __importDefault(require("chalk"));
const error_1 = require("@pnpm/error");
const store_path_1 = require("@pnpm/store-path");
const load_json_file_1 = __importDefault(require("load-json-file"));
const render_help_1 = __importDefault(require("render-help"));
exports.PACKAGE_INFO_CLR = chalk_1.default.greenBright;
exports.INDEX_PATH_CLR = chalk_1.default.hex('#078487');
exports.commandNames = ['find-hash'];
exports.rcOptionsTypes = cliOptionsTypes;
function cliOptionsTypes() {
    return {};
}
exports.cliOptionsTypes = cliOptionsTypes;
function help() {
    return (0, render_help_1.default)({
        description: 'Experimental! Lists the packages that include the file with the specified hash.',
        descriptionLists: [],
        usages: ['pnpm find-hash <hash>'],
    });
}
exports.help = help;
async function handler(opts, params) {
    if (!params || params.length === 0) {
        throw new error_1.PnpmError('MISSING_HASH', '`pnpm find-hash` requires the hash');
    }
    const hash = params[0];
    const storeDir = await (0, store_path_1.getStorePath)({
        pkgRoot: process.cwd(),
        storePath: opts.storeDir,
        pnpmHomeDir: opts.pnpmHomeDir,
    });
    const cafsDir = path_1.default.join(storeDir, 'files');
    const cafsChildrenDirs = fs_1.default.readdirSync(cafsDir, { withFileTypes: true }).filter(file => file.isDirectory());
    const indexFiles = [];
    const result = [];
    cafsChildrenDirs.forEach(({ name: dirName }) => {
        const dirIndexFiles = fs_1.default
            .readdirSync(`${cafsDir}/${dirName}`)
            .filter((fileName) => fileName.includes('-index.json'))
            ?.map((fileName) => `${cafsDir}/${dirName}/${fileName}`);
        indexFiles.push(...dirIndexFiles);
    });
    indexFiles.forEach(filesIndexFile => {
        const pkgFilesIndex = load_json_file_1.default.sync(filesIndexFile);
        for (const [, file] of Object.entries(pkgFilesIndex.files)) {
            if (file?.integrity === hash) {
                result.push({ name: pkgFilesIndex.name ?? 'unknown', version: pkgFilesIndex?.version ?? 'unknown', filesIndexFile: filesIndexFile.replace(cafsDir, '') });
                // a package is only found once.
                return;
            }
        }
        if (pkgFilesIndex?.sideEffects) {
            for (const [, files] of Object.entries(pkgFilesIndex.sideEffects)) {
                for (const [, file] of Object.entries(files)) {
                    if (file?.integrity === hash) {
                        result.push({ name: pkgFilesIndex.name ?? 'unknown', version: pkgFilesIndex?.version ?? 'unknown', filesIndexFile: filesIndexFile.replace(cafsDir, '') });
                        // a package is only found once.
                        return;
                    }
                }
            }
        }
    });
    if (!result.length) {
        throw new error_1.PnpmError('INVALID_FILE_HASH', 'No package or index file matching this hash was found.');
    }
    return result.reduce((acc, { name, version, filesIndexFile }) => {
        acc += `${(0, exports.PACKAGE_INFO_CLR)(name)}@${(0, exports.PACKAGE_INFO_CLR)(version)}  ${(0, exports.INDEX_PATH_CLR)(filesIndexFile)}\n`;
        return acc;
    }, '');
}
exports.handler = handler;
//# sourceMappingURL=findHash.js.map